@@ -6,7 +6,7 @@ use strict;
 use warnings;
 
 use base 'Mojo::Stateful';
-use overload '""' => sub { shift->to_string }, fallback => 1;
+use overload '""' => sub { shift->as_string }, fallback => 1;
 
 use Mojo::Buffer;
 
@@ -15,7 +15,7 @@ __PACKAGE__->attr('buffer',
     default => sub { Mojo::Buffer->new }
 );
 
-*to_string = \&build;
+*as_string = \&build;
 
 my @GENERAL_HEADERS = qw/
     Cache-Control
@@ -122,7 +122,6 @@ sub build {
 }
 
 sub connection { return shift->header('Connection', @_) }
-sub content_disposition { return shift->header('Content-Disposition', @_) }
 sub content_length { return shift->header('Content-Length', @_) }
 sub content_type { return shift->header('Content-Type', @_) }
 sub cookie { return shift->header('Cookie', @_) }
@@ -232,7 +231,7 @@ sub parse {
             }
 
             # Done
-            $self->done;
+            $self->state('done');
             delete $self->{__headers};
             return $self->buffer;
         }
@@ -266,7 +265,7 @@ __END__
 
 =head1 NAME
 
-Mojo::Headers - Headers
+Mojo::Headers - HTTP Headers
 
 =head1 SYNOPSIS
 
@@ -279,7 +278,7 @@ Mojo::Headers - Headers
 
 =head1 DESCRIPTION
 
-L<Mojo::Headers> is a container and parser for HTTP headers.
+L<Mojo::Headers> is a generic container for HTTP headers.
 
 =head1 ATTRIBUTES
 
@@ -291,11 +290,6 @@ implements the following new ones.
     my $connection = $headers->connection;
     $headers       = $headers->connection('close');
 
-=head2 C<content_disposition>
-
-    my $content_disposition = $headers->content_disposition;
-    $headers                = $headers->content_disposition('foo');
-
 =head2 C<content_length>
 
     my $content_length = $headers->content_length;
@@ -365,12 +359,10 @@ the following new ones.
 
     $headers = $headers->add_line('Content-Type', 'text/plain');
 
-=head2 C<to_string>
-
 =head2 C<build>
 
     my $string = $headers->build;
-    my $string = $headers->to_string;
+    my $string = $headers->as_string;
     my $string = "$headers";
 
 =head2 C<header>
