@@ -11,13 +11,13 @@ use Carp qw/carp croak/;
 use File::Basename;
 use File::Spec;
 
-use constant DEBUG => $ENV{MOJO_LOADER_DEBUG} || 0;
-
 __PACKAGE__->attr([qw/base namespace/], chained => 1);
-__PACKAGE__->attr('modules', chained => 1, default => sub { [] });
 
 my $STATS = {};
 
+*inst = \&instantiate;
+*mods = \&modules;
+
 BEGIN {
 
     # Debugger sub tracking
@@ -28,6 +28,18 @@ BEGIN {
     eval 'sub DB::sub' if $] < 5.008007;
 }
 
+sub modules {
+    my ($self, @modules) = @_;
+
+    $self ->{modules} ||= [];
+    $self->{modules} = \@modules if $modules[0];
+
+    # Chained
+    return $self if @modules;
+
+    return @{$self->{modules}};
+}
+
 # Homer no function beer well without.
 sub new {
     my ($class, $namespace) = @_;
@@ -37,59 +49,45 @@ sub new {
     return $self;
 }
 
-sub build {
+sub instantiate {
     my $self = shift;
 
+    # Load
+    $self->load;
+
     # Load and instantiate
     my @instances;
-    foreach my $module (@{$self->modules}) {
-
+    foreach my $module ($self->modules) {
         eval {
             if (my $base = $self->base) {
-                die "SHORTCUT\n" unless $module->isa($base);
+                die unless $module->isa($base);
             }
             my $instance = $module->new(@_);
             push @instances, $instance;
         };
-        croak qq/Couldn't instantiate module "$module": $@/
-          if $@ && $@ ne "SHORTCUT\n";
     }
 
-    return \@instances;
+    return @instances > 1 ? @instances : $instances[0];
 }
 
 sub load {
     my ($self, @modules) = @_;
 
-    $self->modules(\@modules) if @modules;
+    $self->modules(@modules) if $modules[0];
 
-    for my $module (@{$self->modules}) {
+    for my $module ($self->modules) {
 
         # Shortcut
         next if $module->can('isa');
 
         # Load
         eval "require $module";
-        croak qq/Couldn't load module "$module": $@/ if $@;
+        croak qq/Unable to load module "$module": $@/ if $@;
     }
 
     return $self;
 }
 
-sub load_build {
-    my $self  = shift;
-
-    # Instantiate self
-    $self = $self->new unless ref $self;
-
-    # Load
-    $self->load(shift);
-
-    # Build
-    my $instances = $self->build(@_);
-    return $instances->[0];
-}
-
 sub reload {
     while (my ($key, $file) = each %INC) {
 
@@ -102,8 +100,6 @@ sub reload {
         # Modified?
         if ($mtime > $STATS->{$file}) {
 
-            warn "\n$key -> $file modified, reloading!\n" if DEBUG;
-
             # Unload
             delete $INC{$key};
             my @subs = grep { index($DB::sub{$_}, "$file:") == 0 }
@@ -148,6 +144,7 @@ sub search {
             );
             next if -d $full;
             my $name = File::Basename::fileparse($file, qr/\.pm/);
+            $self->{modules} ||= [];
             my $class = "$namespace\::$name";
             push @{$self->{modules}}, $class unless $found{$class};
             $found{$class} ||= 1;
@@ -162,7 +159,7 @@ __END__
 
 =head1 NAME
 
-Mojo::Loader - Loader
+Mojo::Loader - Universal Class Loader
 
 =head1 SYNOPSIS
 
@@ -174,17 +171,17 @@ Mojo::Loader - Loader
       ->search
       ->load
       ->base('Some::Module')
-      ->build;
+      ->instantiate;
 
     # Short
-    my $something = Mojo::Loader->load_build('Some::Namespace');
+    my @instances = Mojo::Loader->new->mods('Some::Namespace')->inst;
 
     # Reload
     Mojo::Loader->reload;
 
 =head1 DESCRIPTION
 
-L<Mojo::Loader> is a class loader and plugin framework.
+L<Mojo::Loader> is a universal class loader and plugin framework.
 
 =head1 ATTRIBUTES
 
@@ -195,8 +192,10 @@ L<Mojo::Loader> is a class loader and plugin framework.
 
 =head2 C<modules>
 
-    my $modules = $loader->modules;
-    $loader     = $loader->modules([qw/MyApp::Foo MyApp::Bar/]);
+    my @modules = $loader->mods;
+    $loader     = $loader->mods(qw/MyApp::Foo MyApp::Bar/);
+    my @modules = $loader->modules;
+    $loader     = $loader->modules(qw/MyApp::Foo MyApp::Bar/);
 
 =head2 C<namespace>
 
@@ -213,29 +212,31 @@ following new ones.
     my $loader = Mojo::Loader->new;
     my $loader = Mojo::Loader->new('MyApp::Namespace');
 
-=head2 C<build>
+=head2 C<instantiate>
 
-    my $instances = $loader->build;
-    my $instances = $loader->build(qw/foo bar baz/);
+    my $first     = $loader->inst;
+    my @instances = $loader->inst;
+    my $first     = $loader->inst(qw/foo bar baz/);
+    my @instances = $loader->inst(qw/foo bar baz/);
+    my $first     = $loader->instantiate;
+    my @instances = $loader->instantiate;
+    my $first     = $loader->instantiate(qw/foo bar baz/);
+    my @instances = $loader->instantiate(qw/foo bar baz/);
+
+Note that only the main package will be instantiated, file contents won't be
+scanned for multiple package declarations.
 
 =head2 C<load>
 
     $loader = $loader->load;
 
-=head2 C<load_build>
-
-    my $instance = Mojo::Loader->load_build('MyApp';
-    my $instance = $loader->load_build('MyApp';
-    my $instance = Mojo::Loader->load_build('MyApp', qw/some args/);
-    my $instance = $loader->load_build('MyApp', qw/some args/);
-
-=head2 C<reload>
-
-    Mojo::Loader->reload;
-
 =head2 C<search>
 
     $loader = $loader->search;
     $loader = $loader->search('MyApp::Namespace');
 
+=head2 C<reload>
+
+    Mojo::Loader->reload;
+
 =cut
\ No newline at end of file
