@@ -6,27 +6,19 @@ use strict;
 use warnings;
 
 use base 'Mojo::Stateful';
-use overload '""' => sub { shift->to_string }, fallback => 1;
+use overload '""' => sub { shift->as_string }, fallback => 1;
 use bytes;
 
 use Carp 'croak';
 use Mojo::Buffer;
 use Mojo::ByteStream;
 use Mojo::Content;
-use Mojo::File::Memory;
-use Mojo::Parameters;
-use Mojo::Upload;
 use Mojo::URL;
 
 __PACKAGE__->attr('buffer',
     chained => 1,
     default => sub { Mojo::Buffer->new }
 );
-__PACKAGE__->attr([qw/
-    build_start_line_callback
-    prepare_builder_callback
-    prepare_parser_callback
-/], chained => 1);
 __PACKAGE__->attr('content',
     chained => 1,
     default => sub { Mojo::Content->new }
@@ -36,11 +28,7 @@ __PACKAGE__->attr([qw/major_version minor_version/],
     default => sub { 1 }
 );
 
-*to_string           = \&build;
-*body_params         = \&body_parameters;
-*build_start_line_cb = \&build_start_line_callback;
-*prepare_builder_cb  = \&prepare_builder_callback;
-*prepare_parser_cb   = \&prepare_parser_callback;
+*as_string = \&build;
 
 # I'll keep it short and sweet. Family. Religion. Friendship.
 # These are the three demons you must slay if you wish to succeed in
@@ -48,15 +36,18 @@ __PACKAGE__->attr([qw/major_version minor_version/],
 sub body {
     my ($self, $content) = @_;
 
+    # External content generator
+    $self->{body} = $content if ref $content eq 'CODE';
+
     # Plain old content
     unless ($self->is_multipart) {
 
         # Get/Set content
         if ($content) {
-            $self->content->file(Mojo::File::Memory->new);
-            $self->content->file->add_chunk($content);
+            $self->content->cache(Mojo::Cache->new);
+            $self->content->cache->add_chunk($content);
         }
-        return $self->content->file->slurp;
+        return $self->content->cache->slurp;
     }
 
     $self->content($content);
@@ -65,47 +56,28 @@ sub body {
 
 sub body_length { shift->content->body_length }
 
-sub body_parameters {
-    my $self = shift;
-    my $params = Mojo::Parameters->new;
-
-    # "x-application-urlencoded"
-    my $content_type = $self->headers->content_type || '';
-    if ($content_type =~ /x-application-urlencoded/i) {
-        my $raw = $self->content->file->slurp;
-
-        # Parse
-        $params->parse($raw);
-
-        return $params;
-    }
-
-    # "multipart/formdata"
-    elsif ($content_type =~ /multipart\/form-data/i) {
-        my $formdata = $self->_parse_formdata;
-
-        # Formdata
-        for my $data (@$formdata) {
-            my $name     = $data->[0];
-            my $filename = $data->[1];
-            my $part     = $data->[2];
-
-            $params->append($name, $part->file->slurp) unless $filename;
-        }
-    }
-
-    return $params;
-}
-
+# Quick Smithers. Bring the mind eraser device!
+# You mean the revolver, sir?
+# Precisely.
 sub build {
     my $self = shift;
     my $message = '';
 
     # Start line
-    $message .= $self->build_start_line;
+    my $offset = 0;
+    while ($offset < $self->start_line_length) {
+        my $chunk = $self->get_start_line_chunk($offset);
+        $offset += length $chunk;
+        $message .= $chunk;
+    }
 
     # Headers
-    $message .= $self->build_headers;
+    $offset = 0;
+    while ($offset < $self->header_length) {
+        my $chunk = $self->get_header_chunk($offset);
+        $offset += length $chunk;
+        $message .= $chunk;
+    }
 
     # Body
     $message .= $self->build_body;
@@ -117,7 +89,27 @@ sub build {
 # It cost 80 million dollars to make.
 # How do you sleep at night?
 # On top of a pile of money, with many beautiful women.
-sub build_body { shift->content->build_body(@_) }
+sub build_body {
+    my $self = shift;
+
+    my $body = '';
+    my $offset = 0;
+    while (1) {
+        my $chunk = $self->get_body_chunk($offset);
+
+        # No content yet, try again
+        next unless defined $chunk;
+
+        # End of content
+        last unless length $chunk;
+
+        # Content
+        $offset += length $chunk;
+        $body .= $chunk;
+    }
+
+    return $body;
+}
 
 sub build_headers {
     my $self = shift;
@@ -128,75 +120,53 @@ sub build_headers {
     # Fix headers
     $self->fix_headers;
 
-    return $self->content->build_headers;
+    return $self->buffer->replace($self->content->build_headers)->as_string;
 }
 
 sub build_start_line {
-    my $self = shift;
-
-    my $startline = '';
-    my $offset = 0;
-    while (1) {
-        my $chunk = $self->get_start_line_chunk($offset);
-
-        # No start line yet, try again
-        next unless defined $chunk;
-
-        # End of start line
-        last unless length $chunk;
-
-        # Start line
-        $offset += length $chunk;
-        $startline .= $chunk;
-    }
-
-    return $startline;
+    croak 'Method "build_start_line" not implemented by subclass';
 }
 
+# B-6
+# You sunk my scrabbleship!
+# This game makes no sense.
+# Tell that to the good men who just lost their lives... SEMPER-FI!
 sub fix_headers {
     my $self = shift;
 
     # Content-Length header is required in HTTP 1.0 messages
-    if ($self->is_version('1.0') && !$self->is_chunked) {
-        $self->headers->content_length($self->body_length)
+    my $length = $self->body_length;
+    if ($self->is_version('1.0') && !$self->is_chunked && $length) {
+        $self->headers->content_length($length)
           unless $self->headers->content_length;
     }
 
     return $self;
 }
 
-sub get_body_chunk { shift->content->get_body_chunk(@_) }
-
-sub get_header_chunk {
+sub get_body_chunk {
     my $self = shift;
 
-    # HTTP 0.9 has no headers
-    return '' if $self->version eq '0.9';
+    return $self->is_chunked
+      ? $self->_get_chunked_body_chunk(@_)
+      : $self->content->get_body_chunk(@_);
+}
 
-    # Fix headers
-    $self->fix_headers;
-
-    $self->content->get_header_chunk(@_);
+sub get_header_chunk {
+    my ($self, $offset) = @_;
+    my $copy = $self->buffer->raw_length
+      ? $self->buffer->empty
+      : $self->build_headers;
+    return substr($copy, $offset, 4096);
 }
 
 sub get_start_line_chunk {
     my ($self, $offset) = @_;
-
-    # Prepare
-    if ($self->prepare_builder_cb && !$self->{_prepared_builder}) {
-        $self->prepare_builder_cb->($self);
-        $self->{_prepared_builder}++;
-    }
-
-    # Start line generator
-    return $self->build_start_line_cb->($self, $offset)
-      if $self->build_start_line_cb;
-
-    my $copy = $self->_build_start_line;
+    my $copy = $self->buffer || $self->build_start_line;
     return substr($copy, $offset, 4096);
 }
 
-sub header_length { shift->content->header_length }
+sub header_length { return length shift->build_headers }
 
 sub headers { shift->content->headers(@_) }
 
@@ -222,12 +192,6 @@ sub is_version {
 sub parse {
     my $self = shift;
 
-    # Prepare
-    if ($self->prepare_parser_cb && !$self->{_prepared_parser}) {
-        $self->prepare_parser_cb->($self);
-        $self->{_prepared_parser}++;
-    }
-
     # Buffer
     $self->buffer->add_chunk(join '', @_) if @_;
 
@@ -240,48 +204,13 @@ sub parse {
     }
 
     # Done
-    $self->done if $self->content->is_done;
+    $self->state('done') if $self->content->is_state('done');
 
     return $self;
 }
 
 sub start_line_length { return length shift->build_start_line }
 
-sub uploads {
-    my $self = shift;
-
-    my $uploads = {};
-    return $uploads unless $self->is_multipart;
-
-    my $formdata = $self->_parse_formdata;
-
-    # Formdata
-    for my $data (@$formdata) {
-        my $name     = $data->[0];
-        my $filename = $data->[1];
-        my $part     = $data->[2];
-
-        next unless $filename;
-
-        my $upload = Mojo::Upload->new;
-        $upload->file($part->file);
-        $upload->filename($filename);
-        $upload->headers($part->headers);
-
-        # Multiple uploads with same name
-        if (exists $uploads->{$name}) {
-            $uploads->{$name} = [$uploads->{$name}]
-              unless ref $uploads->{$name} eq 'ARRAY';
-            push @{$uploads->{$name}}, $upload;
-        }
-
-        # Simple upload
-        else { $uploads->{$name} = $upload }
-    }
-
-    return $uploads;
-}
-
 sub version {
     my ($self, $version) = @_;
 
@@ -302,46 +231,51 @@ sub version {
     return $self;
 }
 
-sub _build_start_line {
-    croak 'Method "_build_start_line" not implemented by subclass';
-}
+sub _get_chunked_body_chunk {
+    my ($self, $offset) = @_;
 
-sub _parse_formdata {
-    my $self = shift;
-
-    # Use cached formdata
-    return $self->{_formdata} if $self->{_formdata};
-
-    my @formdata;
-
-    # Check content
-    my $content = $self->content;
-    return \@formdata unless $content->is_multipart;
-
-    # Walk the tree
-    my @parts;
-    push @parts, $content;
-    while (my $part = shift @parts) {
-
-        # Multipart?
-        if ($part->is_multipart) {
-            unshift @parts, @{$part->parts};
-            next;
-        }
-
-        # "Content-Disposition"
-        my $disposition = $part->headers->content_disposition;
-        next unless $disposition;
-        my ($name)     = $disposition =~ /\ name="?([^\";]+)"?/;
-        my ($filename) = $disposition =~ /\ filename="?([^\"]*)"?/;
-
-        push @formdata, [$name, $filename, $part];
+    # Start
+    unless (defined $self->{_chunk_offset}) {
+        $self->{_chunk_offset} = 0;
+        $self->buffer->empty;
     }
 
-    # Cache formdata
-    $self->{_formdata} = \@formdata;
+    # Buffered?
+    $self->buffer->remove($offset - $self->{_chunk_offset});
+    return $self->buffer->as_string if $self->buffer->buffer_length;
 
-    return \@formdata;
+    # Generate more
+    $self->{_chunk_offset} = $offset;
+    unless ($self->{_chunks_done}) {
+        my $chunk = $self->{body}->($self);
+        return undef unless defined $chunk;
+        my $chunk_length = length $chunk;
+
+        # Trailing headers?
+        my $headers = 1 if ref $chunk && $chunk->isa('Mojo::Headers');
+
+        # End
+        if ($headers || ($chunk_length == 0)) {
+            $self->buffer->add_chunk("\x0d\x0a0\x0d\x0a");
+
+            # Trailing headers
+            $self->buffer->add_chunk("$chunk\x0d\x0a\x0d\x0a") if $headers;
+            $self->{_chunks_done} = 1;
+        }
+
+        # Separator
+        else {
+
+            # First chunk has no leading CRLF
+            $self->buffer->add_chunk("\x0d\x0a") unless $offset == 0;
+
+            # Chunk
+            $self->buffer->add_chunk(
+                sprintf('%x', length $chunk) . "\x0d\x0a$chunk"
+            );
+        }
+    }
+    return $self->buffer->as_string;
 }
 
 1;
@@ -349,7 +283,7 @@ __END__
 
 =head1 NAME
 
-Mojo::Message - Message Base Class
+Mojo::Message - HTTP Message Base Class
 
 =head1 SYNOPSIS
 
@@ -357,7 +291,7 @@ Mojo::Message - Message Base Class
 
 =head1 DESCRIPTION
 
-L<Mojo::Message> is a base class for HTTP messages.
+L<Mojo::Message> is a generic base class for HTTP messages.
 
 =head1 ATTRIBUTES
 
@@ -373,17 +307,6 @@ implements the following new ones.
     my $buffer = $message->buffer;
     $message   = $message->buffer(Mojo::Buffer->new);
 
-=head2 C<build_start_line_cb>
-
-=head2 C<build_start_line_callback>
-
-    my $cb = $message->build_start_line_cb;
-    my $cb = $message->build_start_line_callback;
-
-    $message = $content->build_start_line_callback(sub {
-        return "HTTP/1.1 200 OK\r\n\r\n";
-    });
-
 =head2 C<content>
 
     my $content = $message->content;
@@ -408,28 +331,6 @@ implements the following new ones.
     my $minor_version = $message->minor_version;
     $message          = $message->minor_version(1);
 
-=head2 C<prepare_builder_cb>
-
-=head2 C<prepare_builder_callback>
-
-    my $cb   = $message->prepare_builder_cb;
-    my $cb   = $message->prepare_builder_callback;
-    $message = $message->prepare_builder_callback(sub {
-        my $self = shift;
-        # Do stuff! :)
-    });
-
-=head2 C<prepare_parser_cb>
-
-=head2 C<prepare_parser_callback>
-
-    my $cb   = $message->prepare_parser_cb;
-    my $cb   = $message->prepare_parser_callback;
-    $message = $message->prepare_parser_callback(sub {
-        my $self = shift;
-        # Do stuff! :)
-    });
-
 =head2 C<raw_body_length>
 
     my $raw_body_length = $message->raw_body_length;
@@ -448,19 +349,24 @@ implements the following new ones.
 L<Mojo::Message> inherits all methods from L<Mojo::Stateful> and implements
 the following new ones.
 
+=head2 C<as_string>
+
+    my $string = $message->as_string;
+
 =head2 C<body>
 
     my $string = $message->body;
     $message = $message->body('Hello!');
 
-=head2 C<body_params>
-
-=head2 C<body_parameters>
-
-    my $params = $message->body_params;
-    my $params = $message->body_parameters;
-
-=head2 C<to_string>
+    $counter = 1;
+    $message = $message->body(sub {
+        my $self  = shift;
+        my $chunk = '';
+        $chunk    = "hello world!" if $counter == 1;
+        $chunk    = "hello world2!\n\n" if $counter == 2;
+        $counter++;
+        return $chunk;
+    });
 
 =head2 C<build>
 
@@ -498,7 +404,7 @@ the following new ones.
 
     my $is_chunked = $message->is_chunked;
 
-=head2 C<is_multipart>
+=header2 C<is_multipart>
 
     my $is_multipart = $message->is_multipart;
 
@@ -510,8 +416,4 @@ the following new ones.
 
     $message = $message->parse('HTTP/1.1 200 OK...');
 
-=head2 C<uploads>
-
-    my $uploads = $message->uploads;
-
 =cut
\ No newline at end of file
