@@ -11,8 +11,6 @@ use base 'Mojo::Base';
 # he'd eat you and everyone you care about!
 __PACKAGE__->attr('state', chained => 1, default => sub { 'start' });
 
-sub done { shift->state('done') }
-
 sub error {
     my ($self, $message) = @_;
     return $self->{error} unless $message;
@@ -22,8 +20,6 @@ sub error {
 
 sub has_error { return defined shift->{error} }
 
-sub is_done { return shift->state eq 'done' }
-
 sub is_state {
     my ($self, @states) = @_;
     for my $state (@states) { return 1 if $self->state eq $state }
@@ -35,7 +31,7 @@ __END__
 
 =head1 NAME
 
-Mojo::Stateful - Stateful Base Class
+Mojo::Stateful - State Keeping Base Class
 
 =head1 SYNOPSIS
 
@@ -43,7 +39,7 @@ Mojo::Stateful - Stateful Base Class
 
 =head1 DESCRIPTION
 
-L<Mojo::Stateful> is a base class for state keeping instances.
+L<Mojo::Stateful> is a generic base class for state keeping instances.
 
 =head1 ATTRIBUTES
 
@@ -62,18 +58,10 @@ L<Mojo::Stateful> is a base class for state keeping instances.
 L<Mojo::Stateful> inherits all methods from L<Mojo::Base> and implements the
 following new ones.
 
-=head2 C<done>
-
-    $stateful = $stateful->done;
-
 =head2 C<has_error>
 
     my $has_error = $stateful->has_error;
 
-=head2 C<is_done>
-
-    my $done = $stateful->is_done;
-
 =head2 C<is_state>
 
     my $is_state = $stateful->is_state('writing');
